<?php


namespace Drupal\simplenews_stats\Controller;

use Drupal\Component\Utility\Tags;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\simplenews_stats\SimplenewsStatsTools;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class SimplenewsStatsAdminController
 *
 * @package Drupal\simplenews_stats\Controller
 */
class SimplenewsStatsAutocompleteController extends ControllerBase {

  /**
   * @var \Drupal\Core\Database\Database
   */
  protected $database;

  /**
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * @var \Drupal\simplenews_stats\SimplenewsStatsTools
   */
  protected $simplenewsStatsTools;

  /**
   * SimplenewsStatsAdminController constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   */
  public function __construct(Connection $database, EntityRepositoryInterface $entityRepository, SimplenewsStatsTools $simplenewsStatsTools) {
    $this->database = $database;
    $this->entityRepository = $entityRepository;
    $this->simplenewsStatsTools = $simplenewsStatsTools;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity.repository'),
      $container->get('simplenews_stats.tools')
    );
  }

  /**
   * Route Callback: Autocomplete callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function autocompleteEntityAssociated(Request $request) {

    $string = $request->query->get('q');
    // Get the typed string from the URL, if it exists.
    if (!$string) {
      return new JsonResponse([]);
    }

    $snsi_table = $this->entityTypeManager()
      ->getDefinition('simplenews_stats_item')
      ->getBaseTable();

    $types = $this->database->select($snsi_table, 'snsi')
      ->fields('snsi', ['entity_type'])
      ->distinct()
      ->execute()->fetchCol();

    $select_union = NULL;
    foreach ($types as $entity_type) {
      // Get the entity data table.
      $table = $this->entityTypeManager()
        ->getDefinition($entity_type)
        ->getDataTable();

      // This is the label field, every entity does not defined this key.
      $key = $this->entityTypeManager()
        ->getDefinition($entity_type)
        ->getKey('label');

      // The entity id field.
      $field_id = $this->entityTypeManager()
        ->getDefinition($entity_type)
        ->getKey('id');

      // Escape if there is not enough.
      if (!$table || !$key || !$field_id) {
        continue;
      }

      // Build the union query.
      $select = $this->database->select($table, "{$entity_type}_table");
      // Add the label field.
      $select->addfield("{$entity_type}_table", $key, "{$entity_type}_field_label");
      // Filter on the entity label field.
      $select->condition("{$entity_type}_table.{$key}", "{$string}%", 'LIKE');
      // Limit the query to the entities used.
      $select->leftJoin($snsi_table, 'snsi', "snsi.entity_type='{$entity_type}' && snsi.entity_id={$entity_type}_table.{$field_id}");
      $select->fields('snsi', ['entity_type', 'entity_id']);

      if ($select_union) {
        $select_union->union($select);
      }
      else {
        $select_union = $select;
      }
    }

    $results = [];
    foreach ($select_union->execute()->fetchAll() as $result) {

      $entity = $this->entityTypeManager->getStorage($result->entity_type)
        ->load($result->entity_id);

      $results[] = [
        'value' => $this->simplenewsStatsTools->getEntityLabel($entity, TRUE),
        'label' => $this->simplenewsStatsTools->getEntityLabel($entity, TRUE),
      ];
    }

    return new JsonResponse($results);
  }

  /**
   * Route Callback: Autocomplete callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function autocompleteUser(Request $request) {
    $string = $request->query->get('q');
    // Get the typed string from the URL, if it exists.
    if (!$string) {
      return new JsonResponse([$string]);
    }

    $userStorage = $this->entityTypeManager()->getStorage('user');
    $snsiMapping = $this->entityTypeManager()
      ->getStorage('simplenews_stats_item')
      ->getTableMapping();
    $userMapping = $userStorage->getTableMapping();

    $query = $this->database->select($snsiMapping->getBaseTable(), 'snsi');
    $query->fields('snsi', ['uid']);
    $query->leftJoin($userMapping->getDataTable(), 'user', 'user.uid=snsi.uid');
    $query->condition('user.name', "{$string}%", 'LIKE')
      ->distinct();

    $results = [];
    foreach ($query->execute() as $result) {
      $entity = $userStorage->load($result->uid);

      $results[] = [
        'value' => $this->simplenewsStatsTools->getEntityLabel($entity),
        'label' => $this->simplenewsStatsTools->getEntityLabel($entity),
      ];
    }
    return new JsonResponse($results);
  }

}